      PROGRAM BAND
C
C23456789012345678901234567890123456789012345678901234567890123456789012
C        1         2         3         4         5         6         7
C  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C  *  MAIN PROGRAM:  BAND                                              *
C  *     CALCULATES   1. COLUMN HEIGHTS                                *
C  *                  2. HALF-BAND WIDTH PLUS DIAGONAL                 *
C  *                                                                   *
C  *  LOCATES AND SHIFTS  1. EQUATION NUMBERS FOR ADDITIONAL           *
C  *                    EQUATION USED WHEN LOAD IS EXCLUSIVELY A       *
C  *                    CHANGE IN THERMAL-MOISTURE CONDITIONS.         *
C  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C
      COMMON/RDK1/LW,LR,KEY
C
      DIMENSION TITLE(20,5)
C
C  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C  *  THE FOLLOWING TWO CARDS DETERMINE THE MAXIMUM HIGH SPEED CORE    *
C  *  STORAGE SHARED BY DIMENSIONED VARIABLES IN VARIOUS SUBROUTINES.  *
C  *     TO CHANGE STORAGE, CHANGE BOTH MTOT AND SHARE(MTOT).          *
C  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C
      COMMON SHARE(50000)
      MTOT=50000
c
      open(5,file='band.dat',status='old',err=8888)
      open(6,file='band.out',status='unknown',err=8888)
C
C   ASSIGN TAPE NO.S 5 AND 6 TO INPUT AND OUTPUT FILES RESPECTIVELY
C
      LR=5
      LW=6
C
C           H  E  A  D  I  N  G
C
      WRITE(LW,99)
   99 FORMAT(1H1)
      WRITE(LW,999) 
  999 FORMAT(79(1H*))
      READ(LR,10)((TITLE(I,J),I=1,20),J=1,5)
   10 FORMAT(20A4)
      WRITE(LW,11)((TITLE(I,J),I=1,20),J=1,5)
   11 FORMAT(1X,20A4)
      WRITE(LW,999) 
C
C         G  R  I  D     P  A  R  A  M  E  T  E  R  S
C
      READ(LR,20)NE,NDS,KEY
   20 FORMAT(2I6,12X,I6)
      READ(LR,21)SMH,SMZ
   21 FORMAT(2F12.0)
C
      WRITE(LW,22)NE,NDS,SMH,SMZ,KEY
   22 FORMAT(//,5X,'G R I D   P A R A M E T E R S',/,
     1' NUMBER OF ELEMENTS .........................',I4,/, 
     2' NUMBER OF NODES ............................',I4,/, 
     3' SCALE FOR HORIZONTAL COORD.S  H(I) .........',F12.5,/,
     4' SCALE FOR VERTICAL   COORD.S  Z(I) .........',F12.5,/,
     5' OUTPUT FILE PRINT KEY ......................',I4,/) 
C
C      N O D E - E L E M E N T    I N F O R M A T I O N
C
C     ADDRESSES
      N1=1
      N2=N1+3*NDS
      N3=N2+NDS
C
C          NODE BOUNDARY CONDITION CODES  ID(NDS,3)
C          NODE HORIZONTAL CO-ORDINATES    H(NDS) 
C          NODE VERTICAL CO-ORDINATES      Z(NDS) 
C
C                 ID(NDS,3)  H(NDS)    Z(NDS)
      CALL NEDATA(SHARE(N1),SHARE(N2),SHARE(N3),NDS,NE,SMH,SMZ)
C
C              L O A D    I N F O R M A T I O N
C  O T H E R   T H E N   L O A D S   A T   N O D E   P O I N T S
C
      WRITE(LW,60)
   60 FORMAT(1H1,10X,'L O A D   I N F O R M A T I O N',/,2X,
     *'O T H E R   T H E N   L O A D S   A T   N O D E   P O I N T S',
     *//) 
C
      READ(LR,61)LTYPE
   61 FORMAT(I6)
      WRITE(LW,64)LTYPE
   64 FORMAT(/,10X,'LOAD TYPE = ',I1,' ( = 0, NODE LOADS ONLY)',
     */,23X,' ( = 1, STRAIN APPLIED NORMAL TO ELEMENTS ONLY)',
     */,23X,' ( = 2, CHANGE IN THERMAL-MOISTURE CONDITIONS ONLY)',//) 
      WRITE(LW,65)LTYPE
   65 FORMAT(5X,'LOAD TYPE =',I2,///)
C
C  A S S I G N   E Q U A T I O N   N U M B E R S   TO
C      E A C H   D E G R E E   O F   F R E E D O M
C            STORED IN ARRAY IEQN(NDS,3)
C
C
C     ADDRESSES
      N1=1
      N2=N1+3*NDS
C
C                ID(NDS,3),IEQN(NDS,3)
      CALL EQNUM(SHARE(N1),SHARE(N2),NDS,NEQ,NEQF,LTYPE)
C
C C A L C U L A T E   A D D R E S S E S    U S E D   T O    S T O R E 
C      T H E   G L O B A L   S T I F F N E S S   A R R A Y
C
C  GLOBAL ARRAY SIZE, NCOFS (NO. OF COEFFICIENTS) 
C  GLOBAL ADDRESSES OF DIAGONAL TERMS, MAXA(NEQ1) 
C  GLOBAL HALF BANDWIDTH INCLUDING DIAGONAL, MBAND
C  ELEMENT ARRAY SIZE, NDOFE (DEGREES OF FREEDOM PER ELEMENT)
C  THERMAL-MOISTURE LOAD EQN. NO., NEQF 
C
C          CALCULATE NO. EQN.S PLUS ONE, NEQ1 USED TO
C        STORE DIAGONAL TERMS OF GLOBAL STIFFNESS ARRAY
      NEQ1=NEQ+1
C
C     ADDRESSES
      N1=1
      N2=N1+NDS*3
      N3=N2+NDS*3
      N4=N3+NEQ1
      N5=N4+NEQ1
C
C             IEQN(NDS,3) MAXA(NEQ1) MAX(NEQ1) MIN(NEQ1)
      CALL ADDRS(SHARE(N2),SHARE(N3),SHARE(N4),SHARE(N5),NE,NDS,NEQ1, 
     *                                  MBAND,NCOFS,NDOFE,LTYPE,NEQF) 
C
 8888 STOP
      END 
C***********************************************************************
      SUBROUTINE NEDATA(ID,H,Z,NDS,NE,SMH,SMZ)
C         CALLING PROGRAM
C  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C  *   P R O G R A M                                         *
C  *     READS NODE - ELEMENT DATA FROM TAPE UNIT LR.        *
C  *     CALCULATES ELEMENT AREAS AND STORES NODE NO.S FOR   *
C  *     REFERENCE ON SCRATCH FILE TAPE UNIT NUMBER 1.       *
C  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C
      COMMON/RDK1/LW,LR,KEY
C
C  DIMENSIONED VARIABLES SHARED WITH MAIN PROGRAM COMMON
C
      DIMENSION ID(NDS,3),H(NDS),Z(NDS) 
C
C                  R E A D   N O D E    D A T A
C
C   WRITE HEADER ON OUTPUT FILE
C
      IF((KEY.EQ.2).OR.(KEY.EQ.7))WRITE(LW,10)
   10 FORMAT(1H1,5X,'N O D E    I N F O R M A T I O N',//,
     16X,'(FIXED=1',/,8X,'FREE=0)',/,' NODE  MOTION     CO-ORDINATES',/,
     2'  NO. N  H  Z  ',6X,'H',8X,'Z')
C
C        L O O P    T H R U    A L L   N O D E S
C
      DO 40 I=1,NDS 
C
C   READ NODE DATA FROM INPUT FILE
C
      READ(LR,20)ID(I,1),ID(I,2),ID(I,3),H(I),Z(I)
   20 FORMAT(6X,3I3,2F12.0)
C
C    WRITE NODE DATA ON OUTPUT FILE
C
      IF((KEY.EQ.2).OR.(KEY.EQ.7))WRITE(LW,30)I,ID(I,1),ID(I,2),ID(I,3),
     *                                        H(I),Z(I)
   30 FORMAT(1X,I4,3(1X,I1,1X),1X,2F9.5)
C
   40 CONTINUE
C
C       E N D   N O D E   L O O P
C
C
C        R E A D  -  S T O R E   E L E M E N T   D A T A
C
C         C A L C U L A T E :  RESCALED NODE CO-ORDINATES
C                              ASSOCIATED WITH EACH ELEMENT 
C                           ARE USED TO CALCULATE ELEMENT AREAS
C
C  INITIALIZE TOTAL GRID AREA 
C
      TAREA=0
C
C    WRITE HEADER ON OUTPUT FILE
C
      IF((KEY.EQ.2).OR.(KEY.EQ.7))WRITE(LW,50)
   50 FORMAT(1H1,5X,'E L E M E N T   I N F O R M A T I O N',//,
     1' ELMT------NODES----- PLY',/,
     2'  NO.  1ST  2ND  3RD  ANGLE   AREA')
C
C        CALCULATE PI (RADIANS TO DEGREES CONVERSION)
C
      PI=ACOS(-1.)
C
C    L O O P   T H R U   A L L   E L E M E N T S
C
      DO 90 K=1,NE
C
C    READ ELEMENT DATA FROM INPUT FILE
C
      READ(LR,60)ND1,ND2,ND3,THETA
   60 FORMAT(6X,3I6,F8.2)
C
C    STORE NODES ASSOCIATED WITH EACH ELEMENT ON TAPE 1
C
      WRITE(1)ND1,ND2,ND3
C
C    NODE CO-ORDINATES ASSOCIATED WITH EACH ELEMENT RESCALED
C
      H1=H(ND1)*SMH 
      H2=H(ND2)*SMH 
      H3=H(ND3)*SMH 
      Z1=Z(ND1)*SMZ 
      Z2=Z(ND2)*SMZ 
      Z3=Z(ND3)*SMZ 
C
C    ELEMENT AREA CALCULATED
C
      TERM1=-(-H1+H2)*(Z1+Z2)/2.0
      TERM2=-(-H2+H3)*(Z2+Z3)/2.0
      TERM3=-(-H3+H1)*(Z3+Z1)/2.0
      AREA=TERM1+TERM2+TERM3
C
C     TEST EACH ELEMENT FOR ZERO AREA
C
      IF(AREA.EQ.0.0)WRITE(6,65)K
   65 FORMAT(' ELEMENT NO.',I4,' HAS NO AREA')
C
C    WRITE ELEMENT DATA ON OUTPUT FILE
C
      IF((KEY.EQ.2).OR.(KEY.EQ.7))WRITE(LW,70)K,ND1,ND2,ND3,THETA,AREA
   70 FORMAT(4(1X,I4),1X,F6.2,E11.4)
C
C    CALCULATE TOTAL GRID AREA
C
      TAREA=TAREA+AREA
C
   90 CONTINUE
C
C      E N D   E L E M E N T   L O O P
C
C    WRITE GRID AREA ON OUTPUT FILE
C
      WRITE(LW,100)TAREA
  100 FORMAT(//,5X,'TOTAL GRID AREA=',E10.3)
C
      RETURN
      END 
C************************************************************
***********
      SUBROUTINE EQNUM(ID,IEQN,NDS,NEQ,NEQF,LTYPE)
C         CALLING PROGRAM - MAIN
C  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C  *   P R O G R A M                                             *
C  *     GENERATES EQUATION NUMBERS BY REPLACING THE FREE DISPL  *
C  *     BOUNDARY CONDITION CODE (ID=0) WITH AN EQUATION NUMBER. *
C  *     EQUATION NUMBERS ASSOCIATED WITH EACH FREE NODE DEGREE  *
C  *     OF FREEDOM (DOF) ARE STORED IN ARRAY ID(NDS,3) WHICH IS *
C  *     SHARED WITH MAIN PROGRAM CONNOM SHARE(MTOT)             *
C  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C
      COMMON/RDK1/LW,LR,KEY
C
C   DIMENSIONED VARIABLE SHARED WITH MAIN PROGRAM COMMON
C
      DIMENSION ID(NDS,3),IEQN(NDS,3)
C
C   INITIALIZE VARIABLES DEFINING EQUATION NUMBERS
C
      IEQ=0
C
C   WRITE HEADER ON OUTPUT FILE
C
      IF((KEY.EQ.2).OR.(KEY.EQ.7))WRITE(LW,10)
   10 FORMAT(1H1,13X,'EQUATION NUMBERS',//,5(12X,'NODE MOTION  '),/,
     *           5(' # NODE  NORM  HORIZ VERT'))
C
C   SEQUENTIALLY TEST THE DISPLACEMENT BOUNDARY CONDITION
C   CODES( FIXED=1 OR FREE=0 ) FOR EACH NODE.  THERE ARE
C   THREE POSSIBLE DEGREES OF FREEDOM ASSOCIATED WITH EACH NODE.
C      ID(I,1)=0, I-TH NODE IS FREE TO MOVE NORMAL TO THE H-Z PLANE
C      ID(I,2)=0, I-TH NODE IS FREE TO MOVE IN THE HORIZ. H-DIRECTION 
C      ID(I,3)=0, I-TH NODE IS FREE TO MOVE IN THE VERT.  Z-DIRECTION 
C   IF A NODE IS FREE TO MOVE THEN AN EQUATION NUMBER (IEQN) IS
C   ASSOCIATED WITH THAT NODE DEGREE OF FREEDOM.
C
      DO 40 I=1,NDS 
      DO 20 J=1,3
      IF(ID(I,J))6,5,6
    5 IEQ=IEQ+1
      IEQN(I,J)=IEQ 
      GO TO 20
    6 IEQN(I,J)=0
   20 CONTINUE
   40 CONTINUE
C
C    WRITE EQN. NO.S ON OUTPUT FILE
C
      IF((KEY.EQ.2).OR.(KEY.EQ.7))WRITE(LW,3)(I,(IEQN(I,J),J=1,3),I=1 
     *,NDS)
    3 FORMAT(5(' #',I5,3I6))
C
C      LAST VALUE OF IEQN IS EQUAL TO THE TOTAL NO. OF EQUATIONS
C
      NEQ=IEQ
C
C    WRITE TOTAL NUMBER OF EQUATIONS DUE TO NODAL D.O.F. S
C                ON OUTPUT FILE
C
      WRITE(LW,45)NEQ
   45 FORMAT(//,' TOTAL NO. OF EQN.S DUE TO NODAL DEGREES OF FREEDOM',
     *I5,//)
C
C        IF LOAD TYPE IS MECHANICAL LOADS ONLY
C                    THEN RETURN
C
      IF(LTYPE.LE.1)RETURN
C
C   WHEN LOAD TYPE (LTYPE=2) IS EXCLUSIVELY THERMAL AND/OR MOISTURE
C     ADD ONE ADDITIONAL EQN TO THE SET OF SIMULTANEOUS EQUATIONS
C
      NEQ=NEQ+1
C
C       IN ORDER TO MINIMIZE THE CORE STORAGE REQUIREMENTS THE
C     ADDITIONAL EQUATION IS LOCATED AT VARIOUS POSITIONS OF THE
C                 ASSEMBLED GLOBAL STIFFNESS MATRIX
C      THIS POSTION IS SPECIFIED BY THE VALUE OF NEQF
C
C
C            NEQ = TOTAL NO. OF EQN.S TO BE SOLVED
C           NEQF = EQN. NO. WHICH RELATES UNKNOWN THERMAL OR MOISTURE 
C                 STRAIN (UKEEP) TO THE THERMAL OR MOISTURE LOAD B(NEQF)
C
C
C          ADDITIONAL EQN. NO. NEQF IS LOCATED AS THE LAST EQN.
C        OF THE ASSEMBLED (GLOBAL) STIFFNESS ARRAY SUCH THAT THE
C    COLUMN ORIENTED STORAGE OF THE NO. OF COEFF.S (NCOFS) IS MINIMIZED
C
      NEQF=NEQ
C
C                            ( OR )
C        EQUATION NUMBER NEQF LOCATED IN THE MODDLE OF GLOBAL STIFFNESS
C           IN ORDER TO MINIMIZE THE HALF-BAND-WIDTH OF MATRIX USED
C                      FOR OUT-OF-CORE STORAGE SOLUTION
C
C     NEQF=(NEQ+1)/2
C
C
C    WRITE ADDITIONAL EQUATION NUMBER ON OUTPUT FILE
C
      WRITE(LW,50)NEQF
   50 FORMAT(' ADDITIONAL EQN. FOR THERMAL-MOISTURE LOAD, NEQF=',I6)
C
C     TEST FOR LOCATION OF EQUATION NUMBER NEQF
C
      IF(NEQF.EQ.NEQ)RETURN
C
C   IF NEQF IS EMBEDDED WITHIN THE GLOBAL STIFFNESS ARRAY
C
C      THEN  1. LOCATE EQUATION NO. NEQF IN IEQN(I,J) ARRAY 
C
      DO 55 I=1,NDS 
      DO 55 J=1,3
      IF(NEQF.EQ.IEQN(I,J))ISAVE=I
      IF(NEQF.EQ.IEQN(I,J))JSAVE=J
   55 CONTINUE
C
C        WRITE HEADER ON OUTPUT FILE
C
      IF((KEY.EQ.5).OR.(KEY.EQ.7))WRITE(LW,10)
C
C            2. SHIFT EQN. NO.S IN IEQN(I,J) ARRAY BY ONE
C                  STARTING AT EQUATION NO. NEQF
C
      DO 90 I=ISAVE,NDS
      JS=1
      IF(I.EQ.ISAVE)JS=JSAVE
      DO 70 J=JS,3
      IF(IEQN(I,J))70,70,60
   60 IEQN(I,J)=IEQN(I,J)+1
   70 CONTINUE
   90 CONTINUE
C
C        WRITE SHIFTED EQUATION NO.S ON THE OUTPUT FILE
C
      IF((KEY.EQ.5).OR.(KEY.EQ.7))WRITE(LW,3)
     *       (I,(IEQN(I,J),J=1,3),I=1,NDS)
C
C
      RETURN
      END 
C***********************************************************************
      SUBROUTINE ADDRS(IEQN,MAXA,MAX,MIN,NE,NDS,NEQ1,MBAND,NCOFS,NDOFE,
     *                                                   LTYPE,NEQF)
C        CALLING PROGRAM - MAIN
C   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C   *   P R O G R A M                                                 *
C   *    1. DETERMINES COLUMN HEIGHTS                                 *
C   *    2. DETERMINES ADDRESSES = ( I-TH SUBSCRIPT IN A(I) ) FOR     *
C   *          THE DIAGONAL TERMS IN THE GLOBAL ARRAY A(I)            *
C   *               ADDRESSES STORED IN MAXA(NEQ+1).                  *
C   * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
C
      COMMON/RDK1/LW,LR,KEY
C
C   DIMENSIONED VARIABLES ID AND MAXA SHARED WITH MAIN PROGRAM COMMON 
C
      DIMENSION IEQN(NDS,3),MAXA(NEQ1),MAX(NEQ1),MIN(NEQ1),LM(10)
C
C        CLEAR LM AND MAXA ARRAYS
C
      DO 5 I=1,NEQ1 
      MAX(I)=0
      MIN(I)=0
    5 MAXA(I)=0
      DO 10 I=1,10
   10 LM(I)=0
C
C           F O R   E A C H   E L E M E N T    C H O O S E
C   N U M B E R   O F   D E G R E E S   O F   F R E E D O M (NDOFE)
C
C       DEFINE NDOFE=9  1. FOR A STRAIN APPLIED NORMAL TO THE
C                               ELEMENT H-Z PLANE, SLOAD
C                       2. FOR NODE LOADS OR PRESCRIBED DISPL.S
      NDOFE=9
C
C    EXCLUSIVELY DEFINE
C         NDOFE=10      1. FOR A CHANGE IN TEMPERATURE, TLOAD
C                       2. FOR A CHANGE IN ABSORBED MOISTURE, WLOAD
C
      IF(LTYPE.EQ.2)NDOFE=10
C
C   USING EQN. NO.S STORED IN IEQN(NDS,3) ARRAY TOGETHER WITH NODE
C   NO.S (ND1,ND2,ND3) STORED ON TAPE 1  CALCULATE CONNECTIVITY
C                  LM ARRAYS FOR EACH ELEMENT.
C
      REWIND 1
C
C      L O O P   T H R U   A L L   E L E M E N T S
C
      DO 40 K=1,NE
C
C  READ NODE NO.S ASSOCIATED WITH K-TH ELEMENT
C
      READ(1)ND1,ND2,ND3
C
C       ASSIGN EQUATION NUMBERS TO ELEMENT CONNECTIVITY ARRAY
C
C(EQN ASSOCIATED WITH DOF IN N-DIRECTION OF 1ST NODE)=IEQN(ND1,1)=LM(1)
C(EQN ASSOCIATED WITH DOF IN N-DIRECTION OF 2ND NODE)=IEQN(ND2,1)=LM(2)
C(EQN ASSOCIATED WITH DOF IN N-DIRECTION OF 3RD NODE)=IEQN(ND3,1)=LM(3)
C(EQN ASSOCIATED WITH DOF IN H-DIRECTION OF 1ST NODE)=IEQN(ND1,2)=LM(4)
C(EQN ASSOCIATED WITH DOF IN H-DIRECTION OF 2ND NODE)=IEQN(ND2,2)=LM(5)
C(EQN ASSOCIATED WITH DOF IN H-DIRECTION OF 3RD NODE)=IEQN(ND3,2)=LM(6)
C(EQN ASSOCIATED WITH DOF IN Z-DIRECTION OF 1ST NODE)=IEQN(ND1,3)=LM(7)
C(EQN ASSOCIATED WITH DOF IN Z-DIRECTION OF 2ND NODE)=IEQN(ND2,3)=LM(8)
C(EQN ASSOCIATED WITH DOF IN Z-DIRECTION OF 3RD NODE)=IEQN(ND3,3)=LM(9)
C
      LM(1)=IEQN(ND1,1)
      LM(2)=IEQN(ND2,1)
      LM(3)=IEQN(ND3,1)
      LM(4)=IEQN(ND1,2)
      LM(5)=IEQN(ND2,2)
      LM(6)=IEQN(ND3,2)
      LM(7)=IEQN(ND1,3)
      LM(8)=IEQN(ND2,3)
      LM(9)=IEQN(ND3,3)
C
C             ******* EXCLUSIVELY *******
C  FOR THERMAL - MOISTURE LOADS ONLY INCLUDE EXTRA DOF
C  BY ASSINGING EQN NO. NEQF TO LM(10) FOR EACH ELEMENT
C
      IF(LTYPE.EQ.2)LM(10)=NEQF
C
C       WRITE ELEMENT CONNECTIVITY ARRAYS ON OUTPUT FILE
C
      IF((KEY.EQ.5).OR.(KEY.EQ.7))GO TO 14
      GO TO 19
C
   14 IF(NDOFE.EQ.9)WRITE(LW,15)K,NDOFE,(LM(I),I=1,NDOFE)
      IF(NDOFE.EQ.10)WRITE(LW,16)K,NDOFE,(LM(I),I=1,NDOFE)
   15 FORMAT(' ELEMENT=',I5,' LM(',I1,')=',9(I6,1X))
   16 FORMAT(' ELEMENT=',I5,' LM(',I2,')=',10(I6,1X))
C
C         CALCULATE THE COLUMN HEIGHTS ABOVE THE DIAGONAL
C            OF THE ASSEMBLED GLOBAL STIFFNESS ARRAY
C       (COLUMN HEIGHTS TEMPORARILY STORED IN MAXA ARRAY)
C
C   THIS IS DONE BY: 
C         COMPARING VALUES IN LM ARRAYS FOR ELEMENTS
C             WHICH SHARE A DEGREE OF FREEDOM(DOF)
C     COLUMN HEIGHT = MAXA( SHARED DOF ) - LOWEST DOF IN SHARED LM ARRAY
C
   19 DO 30 I=1,NDOFE
      L=LM(I)
      IF(L.EQ.0)GO TO 30
      DO 20 J=1,NDOFE
      M=LM(J)
      IF(M.EQ.0)GO TO 20
      IF(M.GT.L)GO TO 20
      N=L-M+1
      IF(N.GT.MAXA(L))MAX(L)=L
      IF(N.GT.MAXA(L))MIN(L)=M
      IF(N.GT.MAXA(L))MAXA(L)=N
   20 CONTINUE
   30 CONTINUE
   40 CONTINUE
C
C          E N D   E L E M E N T    L O O P
C
C
C             WRITE MAXIMUM AND MINIMUM EQUATION NO.S
C    ASSOCIATED WITH EACH COLUMN IN THE ASSEMBLED STIFFNESS MATRIX
C
      IF((KEY.EQ.5).OR.(KEY.EQ.7))WRITE(LW,45)(I,MAXA(I),
     *                               MAX(I),MIN(I),I=1,NEQ1)
   45 FORMAT(2('  # COLUMN=',I6,' HEIGHT=',I6,
     *                         ' MAX EQN=',I6,' MIN EQN=',I6,' #'))
C
C   CALCUALTE WIDTH OF HALFBAND PLUS DIAGONAL, MBAND
C
      MBAND=0
      NEQ=NEQ1-1
      DO 55 I=1,NEQ 
      IF(MAXA(I).GT.MBAND)MBAND=MAXA(I) 
   55 CONTINUE
      WRITE(LW,56)MBAND
   56 FORMAT('  WIDTH OF HALFBAND PLUS DIAGONAL, MBAND=',I4)
C
C   USING COLUMN HEIGHTS STORED IN MAXA( NEQ )
C      DETERMINE ADDRESSES = ( I-TH SUBSCRIPT IN A(I) )
C      FOR DIAGONAL TERMS IN THE GLOBAL ARRAY A(I)
C      AND STORE VALUES IN MAXA( NEQ + 1 ).
C
C  FIRST COLUMN DIAGONAL ADDRESS BY DEFINITION IS ONE
C
      NN=1
      MAXA(1)=1
      MAXM1=1
C
C   ADD COLUMN HEIGHTS TO PREVIOUS DIAGONAL ADDRESS
C
      DO 60 I=2,NEQ1
      NN=NN+MAXM1
      MAXM1=MAXA(I) 
      MAXA(I)=NN
   60 CONTINUE
C
C  WRITE GLOBAL ARRAY SIZE AND DIAGONAL ADDRESSES 
C                ON THE OUTPUT FILE
C
      NCOFS=MAXA(NEQ1)
      WRITE(LW,70)NCOFS
   70 FORMAT(' NO. OF COEFF.S  IN GLOBAL STIFFNESS ARRAY, NCOFS=',I6) 
C
      IF((KEY.EQ.5).OR.(KEY.EQ.7))GO TO 75
      RETURN
C
   75 NEQ=NEQ1-1
      DO 80 I=1,NEQ,3
      I1=I+1
      I2=I+2
   80 WRITE(LW,85)I,MAXA(I),I1,MAXA(I1),I2,MAXA(I2)
   85 FORMAT(3('#COLUMN NO.=',I6,'-DIAGONAL ADDRESS=',I6,'#'))
C
      RETURN
      END 
