pro pltxy
;
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; *  THIS PROGRAM PLOTS THE RESULTS FOR A CONSTANT FORCE IMPACTING A 1-D    *
; *  BAR, WHERE THE SERIES SOLUTION (EXACT) IS COMPARED WITH THE F.E.M.     *
; *  CALCULATIONS. THIS PROGRAM WAS WRITTEN BY P.R. HELYIGER AND MODIFIED   *
; *  BY R.D. KRIZ ON 1/1/00. PV-WAVE IS USED TO PLOT RESULS                 *
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;
openr,11,'node003d.dat'
openr,12,'node103d.dat'
time=fltarr(200)
u=fltarr(200)
z40=fltarr(200)
z20=fltarr(200)
;
;    READ F.E.M. RESULTS FROM THE INPUT FILES
;
for i=0,199 do begin
   readf,11,format='(49X,E11.4)',a
   z40(i)=a
   readf,12,format='(49X,E11.4)',b
   z20(i)=b
endfor
;
;   DEFINE SCALING
;
smult=1000.0 
;
;   DEFINE PARAMETERS FOR SERIES SOLUTIONS
;
;     F    ,FORCE
;     RL   ,BAR LENGTH
;     A0   ,BAR CROSSECTION AREA
;     E    ,BAR YOUNG'S MODULUS
;     RO   ,BAR DENSITY
;     A    ,WAVE SPEED IN BAR CALCULATED
;     DT   ,TIME INCREMENT CALCULATED
;
pi=acos(-1.0) 
f=4000.0
rl=40.0
a0=4.0
e=1.0E+07
ro=2.588E-04
a=SQRT(e/ro)
dt=4.0*rl/a/200.0
;
;   Setup Z buffer for making gif file
;
 set_plot,'pm'
 TEK_COLOR
 openzbuff,450,450
 !P.BACKGROUND=255
 !P.COLOR=0
;
;  CALCULATE TERMS FOR SERIES SOLUTION AND PLOT
;
fac=8.0*F*rl/(ro*a0*pi^2*a^2)
;window,1,xsize=600,ysize=600,xpos=100,ypos=300
;
for ii=1,2 do begin 
   x=20.0
   if(ii EQ 1) then x=40.0
   t=0.00
   for n=0,199 do begin
      it=0
      sum=0.0
      time(n)=t
      for i=1,91,2 do begin
         rn=i
         if(it EQ 0) then f1=1.0/rn^2
         if(it EQ 1) then f1=-1.0/rn^2
         it=0
         if(f1 GT 0.0) then it=1
         f2=SIN(rn*pi*X*0.5/rl)
         f3=1.0-cos(rn*pi*a*t*0.5/rl)
         sum=sum+fac*f1*f2*f3
      endfor 
      t=t+dt
      u(n)=sum
   endfor 
;
;   SCALE SERIES SOLUTION FOR PLOTTING
;
   for j=0,199 do begin
      time(j)=time(j)*smult
      u(j)=u(j)*smult
   endfor
;
;  PLOT SERIES SOLUTION 
;
if(ii EQ 1) then plot,time,u,color=0,$
                 Title='Constant Force Step Function',$
                 Xtitle='Time',Ytitle='Displacement',$
                 xrange=[0.0,0.85],yrange=[0.0,8.5],$
                 linestyle=0,charsize=1.5,xcharsize=1.2,$
                 ycharsize=1.2
if(ii EQ 2) then oplot,time,u,linestyle=0,color=0
;
endfor
;
;   SCALE F.E.M. SOLUTION FOR PLOTTING
;
for i=0,199 do begin
   z20(i)=z20(i)*smult
   z40(i)=z40(i)*smult
endfor
;
;  PLOT F.E.M. SOLUTION
;
oplot,time,z40,linestyle=1,color=2
oplot,time,z20,linestyle=1,color=4
;
closezbuff,'xyplt.gif'
;
end 
